/*
  naturals.cc

  Copyright (C) 2015 Taiji Yamada <taiji@aihara.co.jp>
*/
#include <iostream>
#include <string>
#include <libgen.h>
#include <vector>
#include <boost/lexical_cast.hpp>
#include <gmpxx.h>

inline
void mpz_firstprime(mpz_t rop, const mpz_t op)
{
  if (mpz_probab_prime_p(op, 25)) { mpz_set(rop, op); return; }
  mpz_nextprime(rop, op);
}

inline
int mpz_probab_composite_p(const mpz_t n, int reps)
{
  return mpz_cmp_ui(n, 1) != 0 && !mpz_probab_prime_p(n, reps);
}
inline
void mpz_firstcomposite(mpz_t rop, const mpz_t op)
{
  if (mpz_probab_composite_p(op, 25)) { mpz_set(rop, op); return; }
  for (; ; mpz_add_ui(rop, op, 1))
    if (mpz_probab_composite_p(rop, 25))
      break;
}
inline
void mpz_nextcomposite(mpz_t rop, const mpz_t op)
{
  mpz_add_ui(rop, op, 1);
  for (; ; mpz_add_ui(rop, rop, 1))
    if (mpz_probab_composite_p(rop, 25))
      break;
}

inline
mpz_class function_hilbert(const mpz_class i)
{
  return 4*i + 1;
}
inline
mpz_class function_pronic(const mpz_class i)
{
  return i*(i+1);
}
inline
mpz_class function_power_of_two(const unsigned long int i)
{
  mpz_class n;

  mpz_ui_pow_ui(n.get_mpz_t(), 2, i);
  return n;
}
inline
mpz_class function_cullen(const unsigned long int i)
{
  return function_power_of_two(i)*i + 1;
}
inline
mpz_class function_woodall(const unsigned long int i)
{
  return function_power_of_two(i)*i - 1;
}
inline
mpz_class function_triangular(const mpz_class i)
{
  return (i * (i+1))/2;
}
inline
mpz_class function_tetrahedral(const mpz_class i)
{
  return (i * (i+1) * (i+2))/6;
}
inline
mpz_class function_pentatope(const mpz_class i)
{
  return (i * (i+1) * (i+2) * (i+3))/24;
}
inline
mpz_class function_pentagonal(const mpz_class i)
{
  return (i * (i*3-1))/2;
}
inline
bool is_square_triangular(const mpz_class n)
{
  mpz_class r1, r2;

  mpz_sqrtrem(r1.get_mpz_t(), r2.get_mpz_t(), n.get_mpz_t());
  return r2 == 0;
}
inline
mpz_class function_squared_triangular(const mpz_class i)
{
  return ((i*i + i)/2)*((i*i + i)/2);
}
inline
mpz_class function_lazy_caterer(const mpz_class i)
{
  return (i*i + i + 2)/2;
}
inline
mpz_class function_jacobsthal(const unsigned long i)
{
  return (function_power_of_two(i) - ((i%2)?-1:1))/3;
}
inline
mpz_class function_jacobsthal_lucas(const unsigned long i)
{
  return function_power_of_two(i) + ((i%2)?-1:1);
}
inline
mpz_class function_kynea(const unsigned long i)
{
  mpz_class n(function_power_of_two(i) + 1);

  return n*n - 2;
}
inline
mpz_class function_thabit(const unsigned long i)
{
  return 3*function_power_of_two(i) - 1;
}
inline
mpz_class function_fermat(const unsigned long i)
{
  return function_power_of_two(function_power_of_two(i).get_ui()) + 1;
}
inline
mpz_class function_catalan(const unsigned long i)
{
  mpz_class n;

  mpz_bin_uiui(n.get_mpz_t(), 2*i, i);
  n /= i+1;
  return n;
}

bool is_square_free(const mpz_class n)
{
  mpz_class i, sqrt_n;

  mpz_sqrt(sqrt_n.get_mpz_t(), n.get_mpz_t());
  for (i=2; i<=sqrt_n; ++i)
    if (n % (i*i) == 0)
      break;
  return i == sqrt_n+1;
}
bool is_cube_free(const mpz_class n)
{
  mpz_class i, cbrt_n;

  mpz_root(cbrt_n.get_mpz_t(), n.get_mpz_t(), 3);
  for (i=2; i<=cbrt_n; ++i)
    if (n % (i*i*i) == 0)
      break;
  return i == cbrt_n+1;
}

std::vector<mpz_class> divisors(const mpz_class n)
{
  mpz_class d(1), sqrt_n;
  std::vector<mpz_class> q;

  mpz_sqrt(sqrt_n.get_mpz_t(), n.get_mpz_t());
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      q.push_back(d);
      if (d != n/d) q.push_back(n/d);
    }
  return q;
}
mpz_class n_divisors(const mpz_class n)
{
  mpz_class d(1), sqrt_n, c(0);

  mpz_sqrt(sqrt_n.get_mpz_t(), n.get_mpz_t());
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      ++c;
      if (d != n/d) ++c;
    }
  return c;
}
mpz_class sum_divisors(const mpz_class n)
{
  mpz_class d(1), sqrt_n, s(0);

  mpz_sqrt(sqrt_n.get_mpz_t(), n.get_mpz_t());
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      s += d;
      if (d != n/d) s += n/d;
    }
  return s;
}

inline
bool is_abundant(const mpz_class n)
{
  return sum_divisors(n) > n*2;
}
inline
bool is_perfect(const mpz_class n)
{
  return sum_divisors(n) - n == n;
}
inline
bool is_multiperfect(const mpz_class n)
{
  return sum_divisors(n) % n == 0;
}
inline
bool is_deficient(const mpz_class n)
{
  return sum_divisors(n) < n*2;
}
inline
bool is_superperfect(const mpz_class n)
{
  return sum_divisors(sum_divisors(n)) == n*2;
}
inline
bool is_hemiperfect(const mpz_class n, mpz_class &k)
{
  mpz_class s;
  return (s=sum_divisors(n))*2 % n == 0 && (k = s*2/n) % 2 == 1;
}
inline
bool is_refactorable(const mpz_class n)
{
  return n % n_divisors(n) == 0;
}
inline
bool is_arithmetic(const mpz_class n, mpz_class &s, mpz_class &c)
{
  return (s=sum_divisors(n)) % (c=n_divisors(n)) == 0;
}

bool is_harmonic_divisor(const mpz_class n)
{
  std::vector<mpz_class> q = divisors(n);
  mpz_class d(0), c(q[0]);
  size_t i;

  for (i=1; i<q.size(); ++i)
    mpz_lcm(c.get_mpz_t(), c.get_mpz_t(), q[i].get_mpz_t());
  for (i=0; i<q.size(); ++i)
    d += c/q[i];
  return (mpz_class(q.size())*c) % d == 0;
}

std::vector<mpz_class> proper_divisors(const mpz_class n)
{
  mpz_class d(1), sqrt_n;
  std::vector<mpz_class> q;

  mpz_sqrt(sqrt_n.get_mpz_t(), n.get_mpz_t());
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (n != d) q.push_back(d);
      if (n != n/d && d != n/d) q.push_back(n/d);
    }
  return q;
}
mpz_class n_proper_divisors(const mpz_class n)
{
  mpz_class d(1), sqrt_n, c(0);

  mpz_sqrt(sqrt_n.get_mpz_t(), n.get_mpz_t());
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (n != d) ++c;
      if (n != n/d && d != n/d) ++c;
    }
  return c;
}
mpz_class sum_proper_divisors(const mpz_class n)
{
  mpz_class d(1), sqrt_n, s(0);

  mpz_sqrt(sqrt_n.get_mpz_t(), n.get_mpz_t());
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (n != d) s += d;
      if (n != n/d && d != n/d) s += n/d;
    }
  return s;
}

bool is_primitive_abundant(const mpz_class n)
{
  if (!is_abundant(n)) return false;
  std::vector<mpz_class> d = proper_divisors(n);
  std::vector<mpz_class>::iterator it;

  for (it=d.begin(); it!=d.end(); ++it)
    if (!(sum_divisors(*it) < *it*2))
      break;
  return it == d.end();
}
bool is_primitive_abundant_have_no_abundant_proper_divisor(const mpz_class n)
{
  if (!is_abundant(n)) return false;
  std::vector<mpz_class> d = proper_divisors(n);
  std::vector<mpz_class>::iterator it;

  for (it=d.begin(); it!=d.end(); ++it)
    if ((sum_divisors(*it) > *it*2))
      break;
  return it == d.end();
}
bool is_pseudoperfect(const mpz_class n)
{
  std::vector<mpz_class> d = proper_divisors(n);
  mpz_class i, c;
  size_t j;

  mpz_ui_pow_ui(c.get_mpz_t(), 2, d.size());
  for (i=1; i<c; ++i) {
    mpz_class s(0);
    for (j=0; j<d.size(); ++j)
      if (mpz_tstbit(i.get_mpz_t(), j))
	s += d[j];
    if (s == n)
      return true;
  }
  return false;
}
bool is_pseudoperfect(const mpz_class n, std::vector<mpz_class> &d)
{
  mpz_class i, c;
  size_t j;

  d = proper_divisors(n);
  mpz_ui_pow_ui(c.get_mpz_t(), 2, d.size());
  for (i=1; i<c; ++i) {
    mpz_class s(0);
    for (j=0; j<d.size(); ++j)
      if (mpz_tstbit(i.get_mpz_t(), j))
	s += d[j];
    if (s == n)
      return true;
  }
  return false;
}
bool is_weird(const mpz_class n)
{
  if (!(sum_divisors(n) > n*2)) return false;
  std::vector<mpz_class> d = proper_divisors(n);
  mpz_class i, c;

  mpz_ui_pow_ui(c.get_mpz_t(), 2, d.size());
  for (i=1; i<c; ++i) {
    mpz_class s(0);
    for (size_t j=0; j<d.size(); ++j)
      if (mpz_tstbit(i.get_mpz_t(), j))
	s += d[j];
    if (s == n) break;
  }
  return i == c;
}
bool is_practical(const mpz_class n)
{
  std::vector<mpz_class> d = proper_divisors(n);
  mpz_class m, i, c;

  mpz_ui_pow_ui(c.get_mpz_t(), 2, d.size());
  for (m = 1; m < n; ++m) {
    for (i=1; i<c; ++i) {
      mpz_class s(0);
      for (size_t j=0; j<d.size(); ++j)
	if (mpz_tstbit(i.get_mpz_t(), j))
	  s += d[j];
      if (s == m) break;
    }
    if (i == c) break;
  }
  return !(m < n);
}

std::vector<mpz_class> unitary_divisors(const mpz_class n)
{
  mpz_class d(1), sqrt_n;
  std::vector<mpz_class> q;

  mpz_sqrt(sqrt_n.get_mpz_t(), n.get_mpz_t());
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      mpz_class n_d = n/d, gcd;
      mpz_gcd(gcd.get_mpz_t(), n_d.get_mpz_t(), d.get_mpz_t());
      if (gcd == 1) {
	q.push_back(d);
	if (d != 1) q.push_back(n/d);
      }
    }
  return q;
}
mpz_class n_unitary_divisors(const mpz_class n)
{
  mpz_class d(1), sqrt_n, c(0);

  mpz_sqrt(sqrt_n.get_mpz_t(), n.get_mpz_t());
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      mpz_class n_d = n/d, gcd;
      mpz_gcd(gcd.get_mpz_t(), n_d.get_mpz_t(), d.get_mpz_t());
      if (gcd == 1) {
	++c;
	if (d != 1) ++c;
      }
    }
  return c;
}
mpz_class sum_unitary_divisors(const mpz_class n)
{
  mpz_class d(1), sqrt_n, s(0);

  mpz_sqrt(sqrt_n.get_mpz_t(), n.get_mpz_t());
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      mpz_class n_d = n/d, gcd;
      mpz_gcd(gcd.get_mpz_t(), n_d.get_mpz_t(), d.get_mpz_t());
      if (gcd == 1) {
	s += d;
	if (d != 1) s += n/d;
      }
    }
  return s;
}

inline
bool is_unitary_perfect(const mpz_class n)
{
  return n != 1 && sum_unitary_divisors(n) == n;
}

template <typename T>
T subfactorial(const T n)
{
  T f(1), i;

  for (i=1; i<=n; ++i)
    f = i*f + ((i%2)?-1:1);
  return f;
}
template <typename T>
T semifactorial(const T n)
{
  T f(1), i((n % 2 == 1) ? 1 : 2);

  for (; i<=n; i+=2)
    f *= i;
  return f;
}

std::vector<mpz_class> factorize(mpz_class n)
{
  mpz_class f(2);
  std::vector<mpz_class> p;

  for (; f <= n; mpz_nextprime(f.get_mpz_t(), f.get_mpz_t())) {
    while (n % f == 0) {
      p.push_back(f);
      n /= f;
    }
  }
  return p;
}
mpz_class sum_factorize(mpz_class n)
{
  mpz_class f(2), s(0);

  for (; f <= n; mpz_nextprime(f.get_mpz_t(), f.get_mpz_t())) {
    while (n % f == 0) {
      s += f;
      n /= f;
    }
  }
  return s;
}

bool is_primitive_pseudoperfect(mpz_class n)
{
  std::vector<mpz_class> d;
  size_t k;

  if (!is_pseudoperfect(n, d)) return false;
  for (k=0; k<d.size(); ++k) {
    if (mpz_probab_prime_p(d[k].get_mpz_t(), 25)) continue;
    std::vector<mpz_class> pd = proper_divisors(d[k]);
    mpz_class i, c;
    size_t j;
    mpz_ui_pow_ui(c.get_mpz_t(), 2, pd.size());
    for (i=1; i<c; ++i) {
      mpz_class s(0);
      for (j=0; j<pd.size(); ++j)
	if (mpz_tstbit(i.get_mpz_t(), j))
	  s += pd[j];
      if (s == d[k]) break;
    }
    if (i != c) break;
  }
  return k == d.size();
}
bool is_non_prime_unusual(mpz_class n)
{
  std::vector<mpz_class> f = factorize(n);
  mpz_class sqrt_n;

  mpz_sqrt(sqrt_n.get_mpz_t(), n.get_mpz_t());
  return f.size() > 1 && f[f.size()-1] > sqrt_n;
}
bool is_unusual(const mpz_class n)
{
  std::vector<mpz_class> f = factorize(n);
  mpz_class sqrt_n;

  mpz_sqrt(sqrt_n.get_mpz_t(), n.get_mpz_t());
  return f.size() > 0 && f[f.size()-1] > sqrt_n;
}

std::vector<mpz_class> prime_factors(mpz_class n)
{
  mpz_class f(2);
  std::vector<mpz_class> p;

  for (; f <= n; mpz_nextprime(f.get_mpz_t(), f.get_mpz_t())) {
    if (n % f == 0) {
      p.push_back(f);
      n /= f;
    }
    while (n % f == 0)
      n /= f;
  }
  return p;
}

bool is_round_number(const mpz_class n)
{
  mpz_class sqrt_n;
  std::vector<mpz_class> f = prime_factors(n);
  std::vector<mpz_class>::iterator it;

  mpz_sqrt(sqrt_n.get_mpz_t(), n.get_mpz_t());
  for (it=f.begin(); it!=f.end(); ++it)
    if (*it > sqrt_n)
      break;
  return it==f.end();
}
bool is_lucas_carmichael(const mpz_class n)
{
  std::vector<mpz_class> f = prime_factors(n);
  std::vector<mpz_class>::iterator it;

  if (f.size() < 3) return false;
  for (it=f.begin(); it!=f.end(); ++it)
    if ((n+1) % (*it+1) != 0)
      break;
  return it==f.end();
}

mpz_class primorial_number(mpz_class n)
{
  mpz_class p(2), f(1), i(1);

  for (; i<=n; mpz_nextprime(p.get_mpz_t(), p.get_mpz_t()), ++i)
    f *= p;
  return f;
}
mpz_class primorial(mpz_class p)
{
  mpz_class n(2), f(1);

  for (; n<=p; mpz_nextprime(n.get_mpz_t(), n.get_mpz_t()))
    f *= n;
  return f;
}

inline
mpz_class function_euclid(const mpz_class i)
{
  return primorial_number(i) + 1;
}
inline
mpz_class function_kummer(const mpz_class i)
{
  return primorial_number(i) - 1;
}

inline
bool is_ruth_aaron_pair(const mpz_class n)
{
  return sum_factorize(n) == sum_factorize(n+1);
}
bool is_primorial_ruth_aaron_pair(const mpz_class n)
{
  std::vector<mpz_class> f = factorize(n);

  return sum_factorize(n) == sum_factorize(n+1) &&
    n*(n+1) == primorial(f[f.size()-1]);
}

bool is_powerful(const mpz_class n)
{
  mpz_class p(2);

  for (; p<n; mpz_nextprime(p.get_mpz_t(), p.get_mpz_t()))
    if (n % p == 0 && n % (p*p) == 0)
      return true;
  return false;
}
bool is_primary_pseudoperfect(const mpz_class n)
{
  std::vector<mpz_class> d = divisors(n);
  mpz_class s(0);
  std::vector<mpz_class>::iterator it;

  for (it=d.begin(); it!=d.end(); ++it)
    if (mpz_probab_prime_p(it->get_mpz_t(), 25))
      s += n/(*it);
  return n != 1 && s + 1 == n;
}

mpz_class totient_function(mpz_class n)
{
  mpz_class f(2), r(n);

  for (; f <= n; mpz_nextprime(f.get_mpz_t(), f.get_mpz_t())) {
    if (n % f == 0) {
      r -= r / f;
      n /= f;
    }
    while (n % f == 0)
      n /= f;
  }
  return r;
}
mpz_class cototient_function(mpz_class n)
{
  mpz_class f(2), r(n);
  const mpz_class c(n);

  for (; f <= n; mpz_nextprime(f.get_mpz_t(), f.get_mpz_t())) {
    if (n % f == 0) {
      r -= r / f;
      n /= f;
    }
    while (n % f == 0)
      n /= f;
  }
  return c - r;
}
mpz_class mebius_function(mpz_class n)
{
  mpz_class f(2), r(1);

  for (; f <= n; mpz_nextprime(f.get_mpz_t(), f.get_mpz_t())) {
    if (n % f == 0) {
      r *= -1;
      n /= f;
    }
    if (n % f == 0)
      return mpz_class(0);
  }
  return r;
}

inline
mpz_class function_mersenne(const mpz_class p)
{
  mpz_class m;

  mpz_ui_pow_ui(m.get_mpz_t(), 2, p.get_ui());
  return m - 1;
}
inline
mpz_class function_wilson_quotient(const mpz_class p)
{
  mpz_class f;

  mpz_fac_ui(f.get_mpz_t(), p.get_ui() - 1);
  return (f + 1)/p;
}
inline
bool is_wilson_prime(const mpz_class n, const mpz_class p)
{
  return n % p == 0;
}

enum naturals_type {
  natural,
  odd,
  even,
  singly_even,
  doubly_even,
  composite,
  prime,
  sophie_germain_prime,
  interprime,
  twin_prime,
  cousin_primes,
  sexy_primes,
  square_free,
  cube_free,
  perfect_square,
  perfect_power,
  powerful,
  //
  highlycomposite,
  highlyabundant,
  superabundant,
  abundant,
  primitive_abundant,
  primitive_abundant_have_no_abundant_proper_divisor,
  non_prime_unusual,
  //
  pseudoperfect, semiperfect,
  primitive_pseudoperfect, primitive_semiperfect,
  perfect,
  multiperfect,
  deficient,
  superperfect,
  hemiperfect,
  refactorable,
  unitary_perfect,
  primary_pseudoperfect,
  weird,
  practical,
  ruth_aaron_pair,
  primorial_ruth_aaron_pair,
  unusual,
  //
  arithmetic,
  harmonic_divisor,
  round_number,
  lucas_carmichael,
  semiprime,
  sphenic,
  amicable,
  betrothed,
  augumented_amicable,
  friendly,
  //
  mersenne,
  mersenne_prime,
  wilson_quotient,
  wilson_prime,
  //
  fortunate,
  hilbert,
  pronic,
  power_of_two,
  cullen,
  cullen_prime,
  woodall,
  woodall_prime,
  regular,
  factorial_prime,
  euclid,
  euclid_prime,
  kummer,
  kummer_prime,
  primorial_prime,
  triangular,
  triangular_pyramidal, tetrahedral,
  pentatope,
  pentagonal,
  square_triangular,
  squared_triangular,
  lazy_caterer,
  jacobsthal,
  jacobsthal_lucas,
  kynea,
  prime_kynea,
  thabit,
  prime_thabit,
  fermat,
  catalan,
  derangement,
  nontotient,
  noncototient,
  perfect_totient,
  highly_totient,
  highly_cototient,
  odd_semifactrial,
  even_semifactrial,
  //
  fibonacci,
  leonardo,
  lucas,
  pell,
  perrin,
  padovan,
  motzkin,
  schreder_hipparchus,
  sylvester,
  telephone,
  alternating_factorial,
  hyperfactorial,
  exponential_factorial,
};
#define S(x) #x
const char *naturals_labels[] = {
  S(natural),
  S(odd),
  S(even),
  S(singly_even),
  S(doubly_even),
  S(composite),
  S(prime),
  S(sophie_germain_prime),
  S(interprime),
  S(twin_prime),
  S(cousin_primes),
  S(sexy_primes),
  S(square_free),
  S(cube_free),
  S(perfect_square),
  S(perfect_power),
  S(powerful),
  //
  S(highlycomposite),
  S(highlyabundant),
  S(superabundant),
  S(abundant),
  S(primitive_abundant),
  S(primitive_abundant_have_no_abundant_proper_divisor),
  S(non_prime_unusual),
  //
  S(pseudoperfect), S(semiperfect),
  S(primitive_pseudoperfect), S(primitive_semiperfect),
  S(perfect),
  S(multiperfect),
  S(deficient),
  S(superperfect),
  S(hemiperfect),
  S(refactorable),
  S(unitary_perfect),
  S(primary_pseudoperfect),
  S(weird),
  S(practical),
  S(ruth_aaron_pair),
  S(primorial_ruth_aaron_pair),
  S(unusual),
  //
  S(arithmetic),
  S(harmonic_divisor),
  S(round_number),
  S(lucas_carmichael),
  S(semiprime),
  S(sphenic),
  S(amicable),
  S(betrothed),
  S(augumented_amicable),
  S(friendly),
  //
  S(mersenne),
  S(mersenne_prime),
  S(wilson_quotient),
  S(wilson_prime),
  //
  S(fortunate),
  S(hilbert),
  S(pronic),
  S(power_of_two),
  S(cullen),
  S(cullen_prime),
  S(woodall),
  S(woodall_prime),
  S(regular),
  S(factorial_prime),
  S(euclid),
  S(euclid_prime),
  S(kummer),
  S(kummer_prime),
  S(primorial_prime),
  S(triangular),
  S(triangular_pyramidal), S(tetrahedral),
  S(pentatope),
  S(pentagonal),
  S(square_triangular),
  S(squared_triangular),
  S(lazy_caterer),
  S(jacobsthal),
  S(jacobsthal_lucas),
  S(kynea),
  S(prime_kynea),
  S(thabit),
  S(prime_thabit),
  S(fermat),
  S(catalan),
  S(derangement),
  S(nontotient),
  S(noncototient),
  S(perfect_totient),
  S(highly_totient),
  S(highly_cototient),
  S(odd_semifactrial),
  S(even_semifactrial),
  //
  S(fibonacci),
  S(leonardo),
  S(lucas),
  S(pell),
  S(perrin),
  S(padovan),
  S(motzkin),
  S(schreder_hipparchus),
  S(sylvester),
  S(telephone),
  S(alternating_factorial),
  S(hyperfactorial),
  S(exponential_factorial),
};
#undef S

int usage(std::ostream &os, int argc, char *argv[])
{
  (void)argc;
  os << "Usage:\n\t" << basename(argv[0]) << " [-type name] [-l lower] [-u upper]\n";
  os << "\ntype names:" << std::endl;
  for (size_t i=0; i<sizeof(naturals_labels)/sizeof(*naturals_labels); ++i)
    os << "\t" << naturals_labels[i] << std::endl;
  return os == std::cerr ? 1 : 0;
}
int main(int argc, char *argv[])
{
  enum naturals_type type(natural);
  int a;
  mpz_class n, l(1), u(100);

  for (a=1; a<argc; ++a)
    if (std::string(argv[a]) == "-h")
      return usage(std::cout, argc, argv);
    else if (std::string(argv[a]) == "-type" && a+1 < argc) {
      for (size_t i=0; i<sizeof(naturals_labels)/sizeof(*naturals_labels); ++i)
	if (std::string(argv[a+1]) == naturals_labels[i]) {
	  ++a;
	  type = static_cast<naturals_type>(i);
	  break;
	}
      switch (type) {
      case pronic:
      case triangular:
      case triangular_pyramidal: case tetrahedral:
      case pentatope:
      case pentagonal:
      case square_triangular:
      case squared_triangular:
      case jacobsthal:
      case derangement:
      case fibonacci:
      case pell:
      case perrin:
	l = 0;
	break;
      default:
	l = 1;
	break;
      }
    }
    else if (std::string(argv[a]) == "-l" && a+1 < argc)
      l = argv[++a];
    else if (std::string(argv[a]) == "-u" && a+1 < argc)
      u = argv[++a];
    else
      break;

  n = l;
  switch (type) {
  case natural:
    for (; n < u; ++n)
      std::cout << n << std::endl;
    break;
  case odd:
    if (n % 2 == 0) ++n;
    for (; n < u; n+=2)
      std::cout << n << std::endl;
    break;
  case even:
    if (n % 2 == 1) ++n;
    for (; n < u; n+=2)
      std::cout << n << std::endl;
    break;
  case singly_even:
    if (n % 2 == 1) ++n;
    if (n % 4 == 0) n += 2;
    for (; n < u; n+=4)
      std::cout << n << std::endl;
    break;
  case doubly_even:
    n += 4 - n % 4;
    for (; n < u; n+=4)
      std::cout << n << std::endl;
    break;
  case composite:
    mpz_firstcomposite(n.get_mpz_t(), n.get_mpz_t());
    for (; n < u; mpz_nextcomposite(n.get_mpz_t(), n.get_mpz_t()))
      std::cout << n << std::endl;
    break;
  case prime:
    mpz_firstprime(n.get_mpz_t(), n.get_mpz_t());
    for (; n < u; mpz_nextprime(n.get_mpz_t(), n.get_mpz_t()))
      std::cout << n << std::endl;
    break;
  case sophie_germain_prime:
    {
      mpz_firstprime(n.get_mpz_t(), n.get_mpz_t());
      for (; n < u; mpz_nextprime(n.get_mpz_t(), n.get_mpz_t())) {
	mpz_class m(n*2 + 1);
	if (mpz_probab_prime_p(m.get_mpz_t(), 25)) std::cout << n << "\t" << m << std::endl;
      }
    }
    break;
  case interprime:
    {
      mpz_class m;

      if (n < 3) n = 3;
      mpz_firstprime(n.get_mpz_t(), n.get_mpz_t());
      mpz_nextprime(m.get_mpz_t(), n.get_mpz_t());
      for (; m < u; n = m, mpz_nextprime(m.get_mpz_t(), n.get_mpz_t()))
	std::cout << (n + m)/2 << std::endl;
    }
    break;
  case twin_prime:
  case cousin_primes:
  case sexy_primes:
    {
      mpz_class d, c, i;
      switch (type) {
      case twin_prime:			d = 2; c = 2; break;
      case cousin_primes:		d = 4; c = 2; break;
      case sexy_primes: default:	d = 6; c = 2; break;
      }
      mpz_firstprime(n.get_mpz_t(), n.get_mpz_t());
      for (; n < u; mpz_nextprime(n.get_mpz_t(), n.get_mpz_t())) {
	for (i=1; i<c; ++i) {
	  mpz_class m(n + d*i);
	  if (!mpz_probab_prime_p(m.get_mpz_t(), 25)) break;
	}
	if (i==c) {
	  for (i=0; i<c; ++i)
	    std::cout << n + d*i << "\t";
	  std::cout << std::endl;
	}
      }
    }
    break;
  case square_free:
    for (; n < u; ++n)
      if (is_square_free(n))
	std::cout << n << std::endl;
    break;
  case cube_free:
    for (; n < u; ++n)
      if (is_cube_free(n))
	std::cout << n << std::endl;
    break;
  case perfect_square:
    for (; n<u; ++n)
      if (mpz_perfect_square_p(n.get_mpz_t()))
	std::cout << n << std::endl;
    break;
  case perfect_power:
    for (; n<u; ++n)
      if (mpz_perfect_power_p(n.get_mpz_t()))
	std::cout << n << std::endl;
    break;
  case powerful:
    for (; n<u; ++n)
      if (is_powerful(n))
	std::cout << n << std::endl;
    break;
  case highlycomposite:
    {
      mpz_class m(0), c;

      for (n = 1; n < 3 && n < u; ++n)
	if ((c=n_divisors(n)) > m) {
	  if (!(n < l)) std::cout << n << std::endl;
	  m = c;
	}
      for (; n < u; mpz_nextcomposite(n.get_mpz_t(), n.get_mpz_t()))
	if ((c=n_divisors(n)) > m) {
	  if (!(n < l)) std::cout << n << std::endl;
	  m = c;
	}
    }
    break;
  case highlyabundant:
    {
      mpz_class m(0), s;

      for (n = 1; n < 3 && n < u; ++n)
	if ((s=sum_divisors(n)) > m) {
	  if (!(n < l)) std::cout << n << std::endl;
	  m = s;
	}
      for (; n < u; mpz_nextcomposite(n.get_mpz_t(), n.get_mpz_t()))
	if ((s=sum_divisors(n)) > m) {
	  if (!(n < l)) std::cout << n << std::endl;
	  m = s;
	}
    }
    break;
  case superabundant:
    {
      mpz_class m(0), s, o(1);

      for (n = 1; n < 3 && n < u; ++n)
	if ((s=sum_divisors(n))*o > m*n) {
	  if (!(n < l)) std::cout << n << std::endl;
	  m = s;
	  o = n;
	}
      for (; n < u; mpz_nextcomposite(n.get_mpz_t(), n.get_mpz_t()))
	if ((s=sum_divisors(n))*o > m*n) {
	  if (!(n < l)) std::cout << n << std::endl;
	  m = s;
	  o = n;
	}
    }
    break;
  case abundant:
    for (mpz_firstcomposite(n.get_mpz_t(), n.get_mpz_t()); n < u; mpz_nextcomposite(n.get_mpz_t(), n.get_mpz_t()))
      if (is_abundant(n))
	std::cout << n << std::endl;
    break;
  case primitive_abundant:
    for (mpz_firstcomposite(n.get_mpz_t(), n.get_mpz_t()); n < u; mpz_nextcomposite(n.get_mpz_t(), n.get_mpz_t()))
      if (is_primitive_abundant(n))
	std::cout << n << std::endl;
    break;
  case primitive_abundant_have_no_abundant_proper_divisor:
    for (mpz_firstcomposite(n.get_mpz_t(), n.get_mpz_t()); n < u; mpz_nextcomposite(n.get_mpz_t(), n.get_mpz_t()))
      if (is_primitive_abundant_have_no_abundant_proper_divisor(n))
	std::cout << n << std::endl;
    break;
  case non_prime_unusual:
    for (mpz_firstcomposite(n.get_mpz_t(), n.get_mpz_t()); n < u; mpz_nextcomposite(n.get_mpz_t(), n.get_mpz_t()))
      if (is_non_prime_unusual(n))
	std::cout << n << std::endl;
    break;
  case pseudoperfect:
  case semiperfect:
    for (mpz_firstcomposite(n.get_mpz_t(), n.get_mpz_t()); n < u; mpz_nextcomposite(n.get_mpz_t(), n.get_mpz_t()))
      if (is_pseudoperfect(n))
	std::cout << n << std::endl;
    break;
  case primitive_pseudoperfect:
  case primitive_semiperfect:
    for (mpz_firstcomposite(n.get_mpz_t(), n.get_mpz_t()); n < u; mpz_nextcomposite(n.get_mpz_t(), n.get_mpz_t()))
      if (is_primitive_pseudoperfect(n))
	std::cout << n << std::endl;
    break;
  case perfect:
    for (; n < u; ++n)
      if (is_perfect(n))
	std::cout << n << std::endl;
    break;
  case multiperfect:
    for (; n < u; ++n)
      if (is_multiperfect(n))
	std::cout << n << std::endl;
    break;
  case deficient:
    for (; n < u; ++n)
      if (is_deficient(n))
	std::cout << n << std::endl;
    break;
  case superperfect:
    for (; n < u; ++n)
      if (is_superperfect(n))
	std::cout << n << std::endl;
    break;
  case hemiperfect:
    for (; n < u; ++n) {
      mpz_class k;
      if (is_hemiperfect(n, k))
	std::cout << n << "\t" << k << std::endl;
    }
    break;
  case refactorable:
    for (; n < u; ++n)
      if (is_refactorable(n))
	std::cout << n << std::endl;
    break;
  case unitary_perfect:
    for (; n < u; ++n)
      if (is_unitary_perfect(n))
	std::cout << n << std::endl;
    break;
  case primary_pseudoperfect:
    for (; n < u; ++n)
      if (is_primary_pseudoperfect(n))
	std::cout << n << std::endl;
    break;
  case weird:
    for (; n < u; ++n)
      if (is_weird(n))
	std::cout << n << std::endl;
    break;
  case practical:
    for (; n < u; ++n)
      if (is_practical(n))
	std::cout << n << std::endl;
    break;
  case ruth_aaron_pair:
    for (; n < u; ++n)
      if (is_ruth_aaron_pair(n))
	std::cout << n << "\t" << n+1 << std::endl;
    break;
  case primorial_ruth_aaron_pair:
    for (; n < u; ++n)
      if (is_primorial_ruth_aaron_pair(n))
	std::cout << n << "\t" << n+1 << std::endl;
    break;
  case unusual:
    for (; n < u; ++n)
      if (is_unusual(n))
	std::cout << n << std::endl;
    break;
  case arithmetic:
    for (; n < u; ++n) {
      mpz_class s, c;
      if (is_arithmetic(n, s, c))
	std::cout << n << "\t" << s/c << std::endl;
    }
    break;
  case harmonic_divisor:
    for (; n < u; ++n)
      if (is_harmonic_divisor(n))
	std::cout << n << std::endl;
    break;
  case round_number:
    for (; n < u; ++n)
      if (is_round_number(n))
	std::cout << n << std::endl;
    break;
  case lucas_carmichael:
    for (; n < u; ++n)
      if (is_lucas_carmichael(n))
	std::cout << n << std::endl;
    break;
  case semiprime:
    {
      mpz_class i, j;

      for (; n < u; ++n)
	for (i = 2; i < u; mpz_nextprime(i.get_mpz_t(), i.get_mpz_t()))
	  for (j = 2; j < u/i; mpz_nextprime(j.get_mpz_t(), j.get_mpz_t()))
	    if (n == i*j) {
	      std::cout << n << "\t" << i << "\t" << j << std::endl;
	      i = u;
	      break;
	    }
    }
    break;
  case sphenic:
    {
      mpz_class i, j, k;

      for (; n < u; ++n) {
	for (i = 2; i < u; mpz_nextprime(i.get_mpz_t(), i.get_mpz_t()))
	  for (mpz_nextprime(j.get_mpz_t(), i.get_mpz_t()); j < u/i; mpz_nextprime(j.get_mpz_t(), j.get_mpz_t()))
	    for (mpz_nextprime(k.get_mpz_t(), j.get_mpz_t()); k < u/i/j; mpz_nextprime(k.get_mpz_t(), k.get_mpz_t()))
	      if (n == i*j*k) {
		std::cout << n << "\t" << i << "\t" << j << "\t" << k << std::endl;
		i = u;
		j = u/i;
		break;
	      }
      }
    }
    break;
  case amicable:
    for (; n < u; ++n) {
      mpz_class s = sum_divisors(n) - n, m;
      for (m = n + 1; m < u; ++m)
	if (s == m && sum_divisors(m) - m == n)
	  std::cout << n << "\t" << m << std::endl;
    }
    break;
  case betrothed:
    for (; n < u; ++n) {
      mpz_class s = sum_divisors(n) - n - 1, m;
      for (m = n + 1; m < u; ++m)
	if (s == m && sum_divisors(m) - m - 1 == n)
	  std::cout << n << "\t" << m << std::endl;
    }
    break;
  case augumented_amicable:
    for (; n < u; ++n) {
      mpz_class s = sum_divisors(n) - n + 1, m;
      for (m = n + 1; m < u; ++m)
	if (s == m && sum_divisors(m) - m + 1 == n)
	  std::cout << n << "\t" << m << std::endl;
    }
    break;
  case friendly:
    for (; n < u; ++n) {
      mpz_class s1 = sum_divisors(n), c1, d1, m;
      mpz_gcd(c1.get_mpz_t(), s1.get_mpz_t(), n.get_mpz_t());
      s1 /= c1;
      d1 = n/c1;
      for (m = n + 1; m < u; ++m) {
	mpz_class s2 = sum_divisors(m), c2, d2;
	mpz_gcd(c2.get_mpz_t(), s2.get_mpz_t(), m.get_mpz_t());
	s2 /= c2;
	d2 = m/c2;
	if (s1 == s2 && d1 == d2)
	  std::cout << n << "\t" << m << std::endl;
      }
    }
    break;
  case mersenne:
    {
      for (mpz_class i=2; ; mpz_nextprime(i.get_mpz_t(), i.get_mpz_t())) {
	n = function_mersenne(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case mersenne_prime:
    {
      for (mpz_class i=2; ; mpz_nextprime(i.get_mpz_t(), i.get_mpz_t())) {
	n = function_mersenne(i);
	if (!(n < u)) break;
	if (mpz_probab_prime_p(n.get_mpz_t(), 25))
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case wilson_quotient:
    {
      for (mpz_class i=2; ; mpz_nextprime(i.get_mpz_t(), i.get_mpz_t())) {
	n = function_wilson_quotient(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case wilson_prime:
    {
      for (mpz_class i=2; ; mpz_nextprime(i.get_mpz_t(), i.get_mpz_t())) {
	n = function_wilson_quotient(i);
	if (!(n < u)) break;
	if (is_wilson_prime(n, i))
	  if (!(n < l)) std::cout << i << "\t" << n << std::endl;
      }
    }
    break;
  case fortunate:
    {
      mpz_class m, i, p;

      for (i = 1; ; ++i) {
	p = primorial_number(i);
	for (m = 2; ; ++m) {
	  n = p + m;
	  if (!(n < u)) break;
	  if (mpz_probab_prime_p(n.get_mpz_t(), 25)) {
	    if (!(n < l)) std::cout << m << "\t" << n << std::endl;
	    break;
	  }
	}
	if (!(n < u)) break;
      }
    }
    break;
  case hilbert:
    {
      for (mpz_class i=0; ; ++i) {
	n = function_hilbert(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case pronic:
    {
      for (mpz_class i=0; ; ++i) {
	n = function_pronic(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case power_of_two:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_power_of_two(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case cullen:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_cullen(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << "\t" << i << std::endl;
      }
    }
    break;
  case cullen_prime:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_cullen(i);
	if (!(n < u)) break;
	if (mpz_probab_prime_p(n.get_mpz_t(), 25))
	  if (!(n < l)) std::cout << n << "\t" << i << std::endl;
      }
    }
    break;
  case woodall:
    {
      for (unsigned long i=1; ; ++i) {
	n = function_woodall(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << "\t" << i << std::endl;
      }
    }
    break;
  case woodall_prime:
    {
      for (unsigned long i=1; ; ++i) {
	n = function_woodall(i);
	if (!(n < u)) break;
	if (mpz_probab_prime_p(n.get_mpz_t(), 25))
	  if (!(n < l)) std::cout << n << "\t" << i << std::endl;
      }
    }
    break;
  case regular:
    {
#if !0
      for (; n < u; ++n) {
	std::vector<mpz_class> f = prime_factors(n);
	std::vector<mpz_class>::iterator it;
	for (it=f.begin(); it!=f.end(); ++it)
	  if (!(*it == 2 || *it == 3 || *it == 5))
	    break;
	if (it == f.end())
	  std::cout << n << std::endl;
      }
#else // ! in order
      mpz_class m[3];

      for (unsigned long i=0; ; ++i) {
	mpz_ui_pow_ui(m[0].get_mpz_t(), 5, i/9);
	mpz_ui_pow_ui(m[1].get_mpz_t(), 3, (i%9)/3);
	mpz_ui_pow_ui(m[2].get_mpz_t(), 2, (i%3));
	n = m[2]*m[1]*m[0];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
#endif
    }
    break;
  case factorial_prime:
    {
      for (unsigned long i=0; ; ++i) {
	mpz_class f;
	mpz_fac_ui(f.get_mpz_t(), i);
	n = f - 1;
	if (!(n < u)) break;
	if (mpz_probab_prime_p(n.get_mpz_t(), 25))
	  if (!(n < l)) std::cout << n << std::endl;
	n = f + 1;
	if (!(n < u)) break;
	if (mpz_probab_prime_p(n.get_mpz_t(), 25))
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case euclid:
    {
      for (mpz_class i=1; ; ++i) {
	n = function_euclid(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case euclid_prime:
    {
      for (mpz_class i=1; ; ++i) {
	n = function_euclid(i);
	if (!(n < u)) break;
	if (mpz_probab_prime_p(n.get_mpz_t(), 25))
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case kummer:
    {
      for (mpz_class i=1; ; ++i) {
	n = function_kummer(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case kummer_prime:
    {
      for (mpz_class i=1; ; ++i) {
	n = function_kummer(i);
	if (!(n < u)) break;
	if (mpz_probab_prime_p(n.get_mpz_t(), 25))
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case primorial_prime:
    {
      for (mpz_class i=1; ; ++i) {
	n = primorial_number(i) - 1;
	if (!(n < u)) break;
	if (mpz_probab_prime_p(n.get_mpz_t(), 25))
	  if (!(n < l)) std::cout << n << "\t" << i << std::endl;
	n += 2;
	if (!(n < u)) break;
	if (mpz_probab_prime_p(n.get_mpz_t(), 25))
	  if (!(n < l)) std::cout << n << "\t" << i << std::endl;
      }
    }
    break;
  case triangular:
    {
      for (mpz_class i=0; ; ++i) {
	n = function_triangular(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case triangular_pyramidal: case tetrahedral:
    {
      for (mpz_class i=0; ; ++i) {
	n = function_tetrahedral(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case pentatope:
    {
      for (mpz_class i=0; ; ++i) {
	n = function_pentatope(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case pentagonal:
    {
      for (mpz_class i=0; ; ++i) {
	n = function_pentagonal(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case square_triangular:
    {
      for (mpz_class i=0; ; ++i) {
	n = function_triangular(i);
	if (!(n < u)) break;
	if (is_square_triangular(n))
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case squared_triangular:
    {
      for (mpz_class i=0; ; ++i) {
	n = function_squared_triangular(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case lazy_caterer:
    {
      for (mpz_class i=0; ; ++i) {
	n = function_lazy_caterer(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case jacobsthal:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_jacobsthal(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case jacobsthal_lucas:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_jacobsthal_lucas(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case kynea:
    {
      for (unsigned long i=1; ; ++i) {
	n = function_kynea(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case prime_kynea:
    {
      for (unsigned long i=1; ; ++i) {
	n = function_kynea(i);
	if (!(n < u)) break;
	if (mpz_probab_prime_p(n.get_mpz_t(), 25))
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case thabit:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_thabit(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case prime_thabit:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_thabit(i);
	if (!(n < u)) break;
	if (mpz_probab_prime_p(n.get_mpz_t(), 25))
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case fermat:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_fermat(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case catalan:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_catalan(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case derangement:
    {
      for (unsigned long i=0; ; ++i) {
	n = subfactorial(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case nontotient:
    {
      mpz_class i, j;

      for (i=2; ; i+=2) {
	n = i*i;
	if (!(n < u)) break;
	for (j = i; j<n; ++j)
	  if (i == totient_function(j))
	    break;
	if (j == n)
	  if (!(n < l)) std::cout << i << std::endl;
      }
    }
    break;
  case noncototient:
    {
      mpz_class i, j;

      for (i=4; ; i+=2) {
	n = i*i;
	if (!(n < u)) break;
	for (j = i; j<n; ++j)
	  if (i == cototient_function(j))
	    break;
	if (j == n)
	  if (!(n < l)) std::cout << i << std::endl;
      }
    }
    break;
  case perfect_totient:
    {
      mpz_class t, s;

      for (; n < u; ++n) {
	t = n;
	s = 0;
	while ((t = totient_function(t)) > 1)
	  s += t;
	if (n != 1) ++s;
	if (s == n)
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case highly_totient:
    {
      mpz_class i, j, d(0), c;

      for (i=1; ; i++) {
	c = 0;
	n = i*i*2;
	if (!(n < u)) break;
	for (j = i; j<n; ++j)
	  if (i == totient_function(j))
	    ++c;
	if (d < c) {
	  if (!(n < l)) std::cout << i << "\t" << c << std::endl;
	  d = c;
	}
      }
    }
    break;
  case highly_cototient:
    {
      mpz_class i, j, d(0), c;

      for (i=1; ; ++i) {
	c = 0;
	n = i*i*2;
	if (!(n < u)) break;
	for (j = i; j<n; ++j)
	  if (i == cototient_function(j))
	    ++c;
	if (d < c) {
	  if (!(n < l)) std::cout << i << "\t" << c << std::endl;
	  d = c;
	}
      }
    }
    break;
  case odd_semifactrial:
    {
      mpz_class i;

      for (i=1; ; i+=2) {
	n = semifactorial(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case even_semifactrial:
    {
      mpz_class i;

      for (i=0; ; i+=2) {
	n = semifactorial(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case fibonacci:
    {
#if !0
      for (unsigned long i=0; ; ++i) {
	mpz_fib_ui(n.get_mpz_t(), i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
#else
      mpz_class F[] = { 0, 1, };
      size_t t;
      for (t=0; t<sizeof(F)/sizeof(F[0]); ++t)
	if (!(F[t] < l)) std::cout << F[t] << std::endl;
      for (; ; ) {
	n = F[0] + F[1];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(F)/sizeof(F[0]); ++t) F[t] = F[t+1];
	F[t] = n;
      }
#endif
    }
    break;
  case leonardo:
    {
#if !0
      for (unsigned long i=0; ; ++i) {
	mpz_fib_ui(n.get_mpz_t(), i+1);
	n *= 2;
	--n;
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
#else
      mpz_class L[] = { 1, 1, };
      size_t t;
      for (t=0; t<sizeof(L)/sizeof(L[0]); ++t)
	if (!(L[t] < l)) std::cout << L[t] << std::endl;
      for (; ; ) {
	n = L[0] + L[1] + 1;
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(L)/sizeof(L[0]); ++t) L[t] = L[t+1];
	L[t] = n;
      }
#endif
    }
    break;
  case lucas:
    {
#if !0
      for (unsigned long i=0; ; ++i) {
	mpz_lucnum_ui(n.get_mpz_t(), i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
#else
      mpz_class L[] = { 2, 1, };
      size_t t;
      for (t=0; t<sizeof(L)/sizeof(L[0]); ++t)
	if (!(L[t] < l)) std::cout << L[t] << std::endl;
      for (; ; ) {
	n = L[0] + L[1];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(L)/sizeof(L[0]); ++t) L[t] = L[t+1];
	L[t] = n;
      }
#endif
    }
    break;
  case pell:
    {
      mpz_class P[] = { 0, 1, };
      size_t t;
      for (t=0; t<sizeof(P)/sizeof(P[0]); ++t)
	if (!(P[t] < l)) std::cout << P[t] << std::endl;
      for (; ; ) {
	n = P[1]*2 + P[0];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(P)/sizeof(P[0]); ++t) P[t] = P[t+1];
	P[t] = n;
      }
    }
    break;
  case perrin:
    {
      mpz_class P[] = { 3, 0, 2, };
      size_t t;
      for (t=0; t<sizeof(P)/sizeof(P[0]); ++t)
	if (!(P[t] < l)) std::cout << P[t] << std::endl;
      for (; ; ) {
	n = P[1] + P[0];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(P)/sizeof(P[0]); ++t) P[t] = P[t+1];
	P[t] = n;
      }
    }
    break;
  case padovan:
    {
      mpz_class P[] = { 1, 1, 1, };
      size_t t;
      for (t=0; t<sizeof(P)/sizeof(P[0]); ++t)
	if (!(P[t] < l)) std::cout << P[t] << std::endl;
      for (; ; ) {
	n = P[0] + P[1];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(P)/sizeof(P[0]); ++t) P[t] = P[t+1];
	P[t] = n;
      }
    }
    break;
  case motzkin:
    {
      mpz_class P[] = { 1, 1, };
      size_t t;
      for (t=0; t<sizeof(P)/sizeof(P[0]); ++t)
	if (!(P[t] < l)) std::cout << P[t] << std::endl;
      for (mpz_class i=2; ; ++i) {
	n = (P[0]*3*(i-1) + P[1]*(2*i+1))/(2+i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(P)/sizeof(P[0]); ++t) P[t] = P[t+1];
	P[t] = n;
      }
    }
    break;
  case schreder_hipparchus:
    {
      mpz_class S[] = { 1, 1, };
      size_t t;
      for (t=0; t<sizeof(S)/sizeof(S[0]); ++t)
	if (!(S[t] < l)) std::cout << S[t] << std::endl;
      for (mpz_class i=3; ; ++i) {
	n = ((6*i - 9)*S[1] - (i - 3)*S[0])/i;
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(S)/sizeof(S[0]); ++t) S[t] = S[t+1];
	S[t] = n;
      }
    }
    break;
  case sylvester:
    {
      mpz_class S[] = { 2, };
      size_t t;
      for (t=0; t<sizeof(S)/sizeof(S[0]); ++t)
	if (!(S[t] < l)) std::cout << S[t] << std::endl;
      for (mpz_class i=1; ; ++i) {
	n = S[0]*(S[0] - 1) + 1;
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(S)/sizeof(S[0]); ++t) S[t] = S[t+1];
	S[t] = n;
      }
    }
    break;
  case telephone:
    {
      mpz_class T[] = { 1, 1, };
      size_t t;
      for (t=0; t<sizeof(T)/sizeof(T[0]); ++t)
	if (!(T[t] < l)) std::cout << T[t] << std::endl;
      for (mpz_class i=2; ; ++i) {
	n = T[1] + (i - 1)*T[0];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(T)/sizeof(T[0]); ++t) T[t] = T[t+1];
	T[t] = n;
      }
    }
    break;
  case alternating_factorial:
    {
      mpz_class F[] = { 1, };
      size_t t;
      for (t=0; t<sizeof(F)/sizeof(F[0]); ++t)
	if (!(F[t] < l)) std::cout << F[t] << std::endl;
      for (unsigned long i=2; ; ++i) {
	mpz_fac_ui(n.get_mpz_t(), i);
	n -= F[0];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(F)/sizeof(F[0]); ++t) F[t] = F[t+1];
	F[t] = n;
      }
    }
    break;
  case hyperfactorial:
    {
      mpz_class F[] = { 1, };
      size_t t;
      for (t=0; t<sizeof(F)/sizeof(F[0]); ++t)
	if (!(F[t] < l)) std::cout << F[t] << std::endl;
      for (unsigned long i=2; ; ++i) {
	mpz_ui_pow_ui(n.get_mpz_t(), i, i);
	n *= F[0];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(F)/sizeof(F[0]); ++t) F[t] = F[t+1];
	F[t] = n;
      }
    }
    break;
  case exponential_factorial:
    {
      mpz_class F[] = { 1, };
      size_t t;
      for (t=0; t<sizeof(F)/sizeof(F[0]); ++t)
	if (!(F[t] < l)) std::cout << F[t] << std::endl;
      for (unsigned long i=1; ; ++i) {
	mpz_ui_pow_ui(n.get_mpz_t(), i, F[0].get_ui());
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(F)/sizeof(F[0]); ++t) F[t] = F[t+1];
	F[t] = n;
      }
    }
    break;
  default:
    break;
  }

  return 0;
}
